/**********************************************************************
 *
 * collectserver.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: collectserver.cpp 166 1999-02-21 22:35:26Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.1  1999/02/21 22:32:56  rjmcnab

   Initial revision.

 */


#include "collectserver.h"
#include <assert.h>


collectserver::collectserver () {
  configinfo.collection = "null";
}

collectserver::~collectserver () {
}

// configure should be called for each line in the
// configuration files to configure the collection server and everything
// it contains. The configuration should take place just before initialisation.
void collectserver::configure (const text_t &key, const text_tarray &cfgline) {
  // configure the collection server
  if (cfgline.size() >= 1) {
    const text_t &value = cfgline[0];
    if (key == "gsdlhome") configinfo.gsdlhome = value;
    else if (key == "collection") {
      configinfo.collection = value;
      collectinfo.shortInfo.name = value;
    } else if (key == "collectdir") configinfo.collectdir = value;
    else if (key == "host") collectinfo.shortInfo.host = value;
    else if (key == "port") collectinfo.shortInfo.port = value.getint();
    else if (key == "public") {
      if (value == "true") collectinfo.isPublic = true;
      else collectinfo.isPublic = false;
    } else if (key == "beta") {
      if (value == "true") collectinfo.isBeta = true;
      else collectinfo.isBeta = false;
    } else if (key == "builddate") collectinfo.buildDate = value.getint();
    else if (key == "languages") collectinfo.languages = cfgline;
    else if (key == "numdocs") collectinfo.numDocs = value.getint();
    else if (key == "numwords") collectinfo.numWords = value.getint();
    else if (key == "numbytes") collectinfo.numBytes = value.getint();
  }
}

void collectserver::configure (const text_t &key, const text_t &value) {
  text_tarray cfgline;
  cfgline.push_back (value);
  configure(key, cfgline);
}


bool collectserver::init (ostream &/*logout*/) {
  return true;
}



// thecollectserver remains the property of the calling code but
// should not be deleted until it is removed from this list.
void collectservermapclass::addcollectserver (collectserver *thecollectserver) {
  // can't add a null collection server
  assert (thecollectserver != NULL);
  if (thecollectserver == NULL) return;
  
  // can't add an collection server with no collection name
  assert (!(thecollectserver->get_collection_name()).empty());
  if ((thecollectserver->get_collection_name()).empty()) return;

  collectserverptr cptr;
  cptr.c = thecollectserver;
  collectserverptrs[thecollectserver->get_collection_name()] = cptr;
}

// getcollectserver will return NULL if the collectserver could not be found
collectserver *collectservermapclass::getcollectserver (const text_t &collection) {
  // can't find a collection with no name
  assert (!collection.empty());
  if (collection.empty()) return NULL;

  iterator here = collectserverptrs.find (collection);
  if (here == collectserverptrs.end()) return NULL;
  
  return (*here).second.c;
}

