/**********************************************************************
 *
 * statusaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: statusaction.cpp 172 1999-02-25 21:59:02Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.6  1999/02/25 21:59:02  rjmcnab

   Merged sources.

   Revision 1.5  1999/02/21 22:33:58  rjmcnab

   Lots of stuff :-)

   Revision 1.4  1999/02/12 02:40:18  sjboddie

   Added page action

   Revision 1.3  1999/02/11 23:07:00  sjboddie

   extended the status action

   Revision 1.2  1999/02/11 01:24:06  rjmcnab

   Fixed a few compiler warnings.

   Revision 1.1  1999/02/11 01:01:27  rjmcnab

   Initial revision.

 */


#include "statusaction.h"
#include <assert.h>

statusaction::statusaction () {
  disabled = true;
  recpt = NULL;

  // this action uses cgi variable "a"
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "status";
  arg_ainfo.savedarginfo = cgiarginfo::must;

  argsinfo.addarginfo (NULL, arg_ainfo);
}

statusaction::~statusaction () {
}

bool statusaction::check_cgiargs (cgiargsclass &/*args*/, ostream &/*logout*/) {
  // don't want to check anything yet.
  return true;
}

void statusaction::get_cgihead_info (cgiargsclass &/*args*/, response_t &response,
				     text_t &response_data, ostream &/*logout*/) {
  response = content;
  response_data = "text/html";
}

bool statusaction::do_action (cgiargsclass &args, recptproto */*collectproto*/,
			      displayclass &/*disp*/, outconvertclass &outconvert, 
			      ostream &textout, ostream &/*logout*/) {
  textout << outconvert << "<html>\n";
  textout << outconvert << "<head>\n";
  textout << outconvert << "<title>Status information</title>\n";
  textout << outconvert << "</head>\n";
  textout << outconvert << "<body>\n";
  if (disabled) textout << outconvert << "<h2>Status disabled</h2>\n";
  else {
    textout << outconvert << "<h2>Receptionist configuration information</h2>\n";
    
    if (recpt == NULL) {
      textout << outconvert << "The status action does not contain information\n";
      textout << outconvert << "about any receptionists. The method set_receptionist\n";
      textout << outconvert << "was probably not called from the module which instantiated\n";
      textout << outconvert << "this action.\n";

    } else {
      const recptconf &rcinfo = recpt->get_configinfo ();

      textout << outconvert << "<table>\n";
      textout << outconvert << "<tr valign=top><th>gsdlhome</th><td>\"" << rcinfo.gsdlhome
	      << "\"</td></tr>\n";
      textout << outconvert << "<tr valign=top><th>collection</th><td>\"" << rcinfo.collection
	      << "\"</td></tr>\n";
      textout << outconvert << "<tr valign=top><th>collectdir</th><td>\"" << rcinfo.collectdir
	      << "\"</td></tr>\n";
      textout << outconvert << "<tr valign=top><th>httpimg</th><td>\"" << rcinfo.httpimg
	      << "\"</td></tr>\n";
      textout << outconvert << "<tr valign=top><th>gwcgi</th><td>\"" << rcinfo.gwcgi
	      << "\"</td></tr>\n";


      // macrofiles
      textout << outconvert << "<tr valign=top><th>macrofiles</th><td>";
      text_tarray::const_iterator macrohere = rcinfo.macrofiles.begin ();
      text_tarray::const_iterator macroend = rcinfo.macrofiles.end ();
      bool macrofirst = true;
      while (macrohere != macroend) {
	if (!macrofirst) textout << outconvert << ", ";
	macrofirst = false;
	textout << outconvert << "\"" << *macrohere << "\"";
	macrohere++;
      }
      textout << outconvert << "</td></tr>\n";

      // saveconf
      textout << outconvert << "<tr valign=top><th>saveconf</th><td>\"" << rcinfo.saveconf
	      << "\"</td></tr>\n";

      // arguments
      cgiargsinfoclass *rcargsinfo = recpt->get_cgiargsinfo_ptr ();
      if (rcargsinfo != NULL) {
	textout << outconvert << "<tr valign=top><th>arguments</th><td>";

	cgiargsinfoclass::const_iterator argsinfohere = rcargsinfo->begin ();
	cgiargsinfoclass::const_iterator argsinfoend = rcargsinfo->end ();
	bool argsinfofirst = true;
	while (argsinfohere != argsinfoend) {
	  if (!argsinfofirst) textout << outconvert << ", ";
	  argsinfofirst = false;
	  textout << outconvert << "\"" << (*argsinfohere).second.shortname << "\"";
	  argsinfohere++;
	}

	textout << outconvert << "</td></tr>\n";
      }      

      // actions
      actionmapclass *actions = recpt->get_actionmap_ptr();
      if (actions != NULL) {
	textout << outconvert << "<tr valign=top><th>actions</th><td>";

	actionptrmap::iterator actionshere = actions->begin ();
	actionptrmap::iterator actionsend = actions->end ();
	bool actionsfirst = true;
	while (actionshere != actionsend) {
	  if (!actionsfirst) textout << outconvert << ", ";
	  actionsfirst = false;
	  assert ((*actionshere).second.a != NULL);
	  if ((*actionshere).second.a != NULL) {
	    textout << outconvert << "\"" << (*actionshere).second.a->get_action_name() << "\"";
	  }
	  actionshere++;
	}
	
	textout << outconvert << "</td></tr>\n";
      }
      
      // protocols
      recptprotolistclass *protocols = recpt->get_recptprotolist_ptr ();
      if (protocols != NULL) {
	textout << outconvert << "<tr valign=top><th>protocols</th><td>";

	recptprotolistclass::iterator protohere = protocols->begin ();
	recptprotolistclass::iterator protoend = protocols->end ();
	bool protofirst = true;
	while (protohere != protoend) {
	  if (!protofirst) textout << outconvert << ", ";
	  protofirst = false;
	  if ((*protohere).p != NULL) {
	    textout << outconvert << "\"" << (*protohere).p->get_protocol_name() << "\"";
	  }
	  protohere++;
	}
	
	textout << outconvert << "</td></tr>\n";
      }

      // converters
      convertinfoclass *converters = recpt->get_convertinfo_ptr ();
      if (converters != NULL) {
	textout << outconvert << "<tr valign=top><th>converters</th><td>";

	convertinfoclass::iterator converthere = converters->begin ();
	convertinfoclass::iterator convertend = converters->end ();
	bool convertfirst = true;
	while (converthere != convertend) {
	  if (!convertfirst) textout << outconvert << ", ";
	  convertfirst = false;
	  textout << outconvert << "\"" << (*converthere).second.name << "\"";
	  converthere++;
	}
	
	textout << outconvert << "</td></tr>\n";
      }


      textout << outconvert 
	      << "</table>\n"
	      << "<hr>\n"
	      << "<h2>Argument information</h2>\n"
	      << "<table>";

      // argument information
      textout << outconvert << "<tr valign=top><th>short name</th><th>long name</th>"
	      << "<th>multiple char?</th>"
	      << "<th>default</th><th>default status</th><th>saved args</th>"
	      << "<th>current value</th></tr>\n";

      cgiargsclass::const_iterator argshere = args.begin();
      cgiargsclass::const_iterator argsend = args.end();
      cgiarginfo *ainfo;
      
      while (argshere != argsend) {
	const text_t &aname = (*argshere).first;
	textout << outconvert 
		<< "<tr valign=top><td>" << aname << "</td>\n";
	  
	  if ((rcargsinfo != NULL) && 
	      ((ainfo=rcargsinfo->getarginfo(aname)) != NULL)) {
	    textout << outconvert << "<td>" << ainfo->longname << "</td>\n";
	    if (ainfo->multiplechar) textout << outconvert << "<td>yes</td>\n";
	    else textout << outconvert << "<td>no</td>\n";
	    textout << outconvert << "<td>" << ainfo->argdefault << "</td>\n";
	    switch (ainfo->defaultstatus) {
	    case cgiarginfo::none: textout << outconvert << "<td>none</td>\n"; break;
	    case cgiarginfo::weak: textout << outconvert << "<td>weak</td>\n"; break;
	    case cgiarginfo::good: textout << outconvert << "<td>good</td>\n"; break;
	    case cgiarginfo::config: textout << outconvert << "<td>config</td>\n"; break;
	    case cgiarginfo::imperative: textout << outconvert << "<td>imperative</td>\n"; break;
	    }
	    switch (ainfo->savedarginfo) {
	    case cgiarginfo::mustnot: textout << outconvert << "<td>mustnot</td>\n"; break;
	    case cgiarginfo::can: textout << outconvert << "<td>can</td>\n"; break;
	    case cgiarginfo::must: textout << outconvert << "<td>must</td>\n"; break;
	    }
	  } else {
	    textout << outconvert << "<td colspan=5></td>\n";
	  }
	
	textout << outconvert << "<td>\"" << (*argshere).second << "\"</td></tr>\n";

	argshere ++;
      }
      
      textout << outconvert 
	      << "</table>\n"
	      << "<hr>\n"
	      << "<h2>Action information</h2>\n"
	      << "<table>";

      // action information
      if (actions != NULL) {
	textout << outconvert 
		<< "<tr><th>action name</th><th>cgi arguments</th></tr>\n";
	
	actionptrmap::iterator actionshere = actions->begin ();
	actionptrmap::iterator actionsend = actions->end ();
	while (actionshere != actionsend) {
	  assert ((*actionshere).second.a != NULL);
	  if ((*actionshere).second.a != NULL) {
	    textout << outconvert 
		    << "<tr><td>" << (*actionshere).second.a->get_action_name()
		    << "</td><td>";

	    cgiargsinfoclass argsinfo = (*actionshere).second.a->getargsinfo();
	    cgiargsinfoclass::const_iterator argsinfohere = argsinfo.begin ();
	    cgiargsinfoclass::const_iterator argsinfoend = argsinfo.end ();
	    bool aifirst = true;
	    while (argsinfohere != argsinfoend) {
	      if (!aifirst) textout << outconvert << ", ";
	      aifirst = false;
	      textout << outconvert << (*argsinfohere).second.shortname;
	      argsinfohere++;
	    }

	    textout << outconvert << "</td></tr>\n";
	  }
	  actionshere++;
	}
      }
	
      textout << outconvert << "</table><hr>\n";
    }
  }
  
  ifstream initin (GSDL_GSDLHOME "/etc/initout.txt");
  if (initin) {
    textout << outconvert << "<p>The initialisation error log, " GSDL_GSDLHOME "/etc/initout.txt, contains the\n";
    textout << outconvert << "following information:\n\n";
    text_t errorpage = "<p><pre>\n";

    char c;
    initin.get(c);
    while (!initin.eof ()) {
      errorpage.push_back(c);
      initin.get(c);
    }
    
    errorpage += "</pre>\n";
    initin.close();
    textout << outconvert << errorpage;

  } else {
    textout << outconvert << "Couldn't read initialisation error log, " GSDL_GSDLHOME "/etc/initout.txt.\n";
  }

  ifstream errin (GSDL_GSDLHOME "/etc/errout.txt");
  if (initin) {
    textout << outconvert << "<p>The error log, " GSDL_GSDLHOME "/etc/errout.txt, contains the\n";
    textout << outconvert << "following information:\n\n";
    text_t errorpage = "<p><pre>\n";

    char c;
    errin.get(c);
    while (!errin.eof ()) {
      errorpage.push_back(c);
      errin.get(c);
    }
    
    errorpage += "</pre>\n";
    errin.close();
    textout << outconvert << errorpage;

  } else {
    textout << outconvert << "Couldn't read error log, " GSDL_GSDLHOME "/etc/errout.txt.\n";
  }

  textout << outconvert << "</body>\n";
  textout << outconvert << "</html>\n";

  return true;
}

void statusaction::configure (const text_t &key, const text_tarray &cfgline) {
  if ((key == "status") && (cfgline.size() == 1) && 
      (cfgline[0] == "true" || cfgline[0] == "on" || cfgline[0] == "enabled")) {
    disabled = false;
  } else {
    // call the parent class to deal with the things which
    // are not dealt with here
    action::configure (key, cfgline);
  }
}
